%%%
%%% Author:
%%%   Andreas Franke <afranke@ags.uni-sb.de>
%%%
%%% Copyright:
%%%   Andreas Franke, 2000
%%%
%%% Last change:
%%%   $Date $ by $Author $
%%%   $Revision $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

%%
%% buffered versions of Open.{file,socket,pipe}
%%

functor

import
   Open(file:   File
        socket: Socket
        pipe:   Pipe)
   
export
   'class': Buffer
   
   file:   BufFile
   socket: BufSocket
   pipe:   BufPipe

prepare
   
   class Stack
      prop
         locking
      attr
         S: nil
         
      meth push(X)
         lock
            S<-X|@S
         end
      end
      meth pop(?X)
         lock
            !X|R=@S in
            S<-R
         end
      end
   end
   
   class Buffer from Stack
      prop
         locking
      attr
         Buf: ''

      meth reset(VS<='')
         lock
            Buf <- VS
         end
      end
      meth append(VS)
         lock
            Buf <- @Buf#VS
         end
      end
      meth getAll(?VS)
         lock
            VS = @Buf
            Buffer, reset
         end
      end
      %%
      meth push
         lock
            VS = Buffer, getAll($) in
            Stack, push(VS)
         end
      end
      meth pop(?VS)
         lock
            VS = @Buf
            Buf <- Stack, pop($)
         end
      end
   end

define
   
   class BufFile from Buffer File 
      prop locking
      meth write(vs:VS)
         lock
            Buffer, append(VS)
         end
      end
      meth flushBuffer
         lock
            VS = Buffer, getAll($) in
            File, write(vs:VS)
         end
      end
      meth close
         lock
            BufFile, flushBuffer
            File, close
         end
      end
   end

   class BufSocket from Buffer Socket
      prop locking
      meth write(vs:VS)
         lock
            Buffer, append(VS)
         end
      end
      meth flushBuffer
         lock
            VS = Buffer, getAll($) in
            Socket, write(vs:VS)
         end
      end
      meth shutDown(...) = ShutDown
         lock
            BufSocket, flushBuffer
            Socket, ShutDown
         end
      end
   end

   class BufPipe from Buffer Pipe
      prop locking
      meth write(vs:VS)
         lock
            Buffer, append(VS)
         end
      end
      meth flushBuffer
         lock
            VS = Buffer, getAll($) in
            Pipe, write(vs:VS)
         end
      end
      meth close(...) = Close
         lock
            BufPipe, flushBuffer
            Pipe, Close
         end
      end
   end
end


