%%%
%%% Author:
%%%   Andreas Franke <afranke@ags.uni-sb.de>
%%%
%%% Copyright:
%%%   Andreas Franke, 2000
%%%
%%% Last change:
%%%   $Date $ by $Author $
%%%   $Revision $
%%%
%%% This file is part of MathWeb
%%%    http://www.mathweb.org/mathweb
%%%
%%% See the file "LICENSE" or
%%%    http://www.mathweb.org/mathweb/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

%%
%% This is a modified version of Open.html for use with objects  
%%
%% Derived from mozart/share/lib/op/Open.oz
%%
%% Original Copyright:

%%%
%%% Author:
%%%   Christian Schulte <schulte@ps.uni-sb.de>
%%%
%%% Copyright:
%%%   Christian Schulte, 1997
%%%
%%% Last change:
%%%   $Date: 2001/06/11 06:17:23 $ by $Author: afranke $
%%%   $Revision: 1.11 $
%%%
%%% This file is part of Mozart, an implementation
%%% of Oz 3
%%%    http://www.mozart-oz.org
%%%
%%% See the file "LICENSE" or
%%%    http://www.mozart-oz.org/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

functor
   
import
   Type(ask)
   Buffered(file socket pipe) at 'x-ozlib://mathweb/html/Buffered.ozf'
   
export
   xml:      Xml
   'class':  Html
   document: HtmlDoc
   
   file:   HtmlFile
   socket: HtmlSocket
   pipe:   HtmlPipe

   escapeS:  EscapeHtml
   escapeVS: EscapeVS
   escape:   EscapeVS

   escapeJS: EscapeJS
   
define

   fun {EscapeHtml S}
      case S of nil then nil
      [] C|Sr then
         case C
         of &< then &&|&l|&t|&;|{EscapeHtml Sr}
         [] &> then &&|&g|&t|&;|{EscapeHtml Sr}
         [] && then &&|&a|&m|&p|&;|{EscapeHtml Sr}
         [] &" then &&|&q|&u|&o|&t|&;|{EscapeHtml Sr}
         else C|{EscapeHtml Sr}
         end
      end
   end
   fun {EscapeVS VS}
      {EscapeHtml {VirtualString.toString VS}}
   end

   fun {EscapeJS S}
      case S of nil then nil
      [] C|Sr then
         case C
         of &" then &\\|&"|{EscapeJS Sr}
         [] &' then &\\|&'|{EscapeJS Sr}
         [] &\\ then &\\|&\\|{EscapeJS Sr}
         [] &\n then &\\|&n|{EscapeJS Sr}
         [] &\r then &\\|&r|{EscapeJS Sr}
         [] &\t then &\\|&t|{EscapeJS Sr}
         [] 0 then &\\|&0|{EscapeJS Sr} %% this only prevents JS errors
         else C|{EscapeJS Sr}
         end
      end
   end
      
   %%
   %% the following is modified code from the Open module
   %%
   local
      %% HtmlTable.oz is unchanged
      \insert 'HtmlTable.oz'
   
      %% GetOptions is unchanged
      fun {GetOptions As N ?M}
         case As of nil then M=N nil
         [] A|Ar then
            if {IsInt A} then {GetOptions Ar N+1 M}
            else M=N As
            end
         end
      end      
      %% BuildOptions is unchanged
      fun {BuildOptions As Tag}
         case As of nil then ''
         [] A|Ar then ' '#A#'="'#Tag.A#'"'#{BuildOptions Ar Tag}
         end
      end

      local
         fun {NoInt X}
            {IsInt X}==false
         end
      in
         fun {AllAttribs Tag}
            {Filter {Arity Tag} NoInt}
         end
      end
      fun {Verb L}
         case L of nil then "nil" else L end
      end
   in
      %% This class requires the existence of a write(vs:...) method.
      %% It must be mixed with one of Open.file/socket/pipe etc.
      class Xml

         meth startTag(T attribs:Attribs<={AllAttribs T})
            L = {Label T} in
            {self write(vs:'<'#{Verb L}#{BuildOptions Attribs T}#'>')}
         end
         meth endTag(T)
            L = {Label T} in
            {self write(vs:'</'#{Verb L}#'>')}
         end
         meth emptyTag(T attribs:Attribs<={AllAttribs T})
            L = {Label T} in
            {self write(vs:'<'#{Verb L}#{BuildOptions Attribs T}#' />')}
         end
         %%
         meth tagBody(N Tag)
            {For 1 N 1
             proc {$ I} {self tag(Tag.I)} end}
         end
         meth tagTuple(Tag)
            case {Label Tag}
            of '#' then
               {Record.forAll Tag
                proc {$ T} {self tag(T)} end}
            [] '|' then
               {self write(vs: Tag)}
            [] L then
               case Tag %% new
               of 'self'(M) then {self M}
               [] ','(C M)  then C, M
               [] nil then skip
               [] '' then skip
               else
                  W = {Width Tag} in
                  if W>0 then
                     {self startTag(L)}
                     {self tagBody(W Tag)}
                     {self endTag(L)}
                  else
                     {self emptyTag(L)}
                  end
               end
            end
         end
         meth tagRecord(Tag)
            N As={GetOptions {Arity Tag} 0 ?N}
         in
            if N>0 then
               {self startTag(Tag attribs:As)}
               {self tagBody(N Tag)}
               {self endTag(Tag)}
            else
               {self emptyTag(Tag attribs:As)}
            end
         end
         meth tagProcedure(P)
            case {Procedure.arity P}
            of 0 then {P} %% new
            [] 1 then {self tag({P})}
            else % raise type error
               {Type.ask.'procedure/1' P}
            end
         end
         meth tag(Tag)
            if {IsTuple Tag} then
               Xml, tagTuple(Tag) 
            elseif {IsRecord Tag} then
               Xml, tagRecord(Tag)   
            elseif {IsProcedure Tag} then
               Xml, tagProcedure(Tag)
            else
               {self write(vs:Tag)}
            end
         end
      end
      
      class Html from Xml
                    
         meth tagRecord(Tag)
            L={Label Tag} in
            if {HtmlTable.isTag L} then
               N As={GetOptions {Arity Tag} 0 ?N}
            in
               {self startTag(Tag attribs:As)}
               {self tagBody(N Tag)}
               if {HtmlTable.isNonFinalTag L}==false then
                  {self endTag(L)}
               end
            else
               {self write(vs:Tag)}
            end
         end
         meth tagTuple(Tag)
            L={Label Tag} in
            if {HtmlTable.isTag L} then
               {self startTag(L)}      
               {self tagBody({Width Tag} Tag)}
               if {HtmlTable.isNonFinalTag L}==false then
                  {self endTag(L)}
               end
            else
               %% was: Xml, tagTuple(Tag)
               
               case {Label Tag}
               of '#' then
                  {Record.forAll Tag
                   proc {$ T} {self tag(T)} end}
               [] '|' then
                  {self write(vs: Tag)}
               [] L then
                  case Tag %% new
                  of 'self'(M) then {self M}
                  [] ','(C M)  then C, M
                  [] nil then skip
                  [] '' then skip
                  else
                     {self write(vs:L)}
                  end
               end
            end
         end
         /** This is _the_ public method! */
         meth tag(Tag)
            if {IsTuple Tag} then
               Html, tagTuple(Tag) 
            elseif {IsRecord Tag} then
               Html, tagRecord(Tag)   
            elseif {IsProcedure Tag} then
               Html, tagProcedure(Tag)
            else
               {self write(vs:Tag)}
            end
         end
      end
   end
   
   class HtmlDoc from Html
                    
      meth writeDoc(title:   Title   <= "Untitled"
                    style:   Style   <= "ozdoc.css"
                    margin:  Margin  <= ""
                    heading: Heading <= ""
                    content: Content <= '')
         {self write(vs:'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">\n')}
         Html, tag(html(head('self'(Head(Title Style)))#
                        body('self'(Body(Margin Heading Content)))))
      end
      meth Head(Title Style)
         Html, tag(title(Title)#"\n")
         Html, tag(link(href:Style rel:"stylesheet" type:"text/css")#"\n")
      end
      meth Body(Margin Heading Content)
         Html, tag(p('class':margin Margin)#"\n")
         Html, tag(Heading#"\n")
         Html, tag(Content)
      end
   end

   HtmlFile   = {ByNeed fun {$} class $ from HtmlDoc Buffered.file   end end}
   HtmlSocket = {ByNeed fun {$} class $ from HtmlDoc Buffered.socket end end}
   HtmlPipe   = {ByNeed fun {$} class $ from HtmlDoc Buffered.pipe   end end}
end

